@@ -71,11 +71,11 @@ add::add(const ex & lh, const ex & rh)
 	GINAC_ASSERT(is_canonical());
 }
 
-add::add(const exvector & v, bool hold)
+add::add(const exvector & v, bool do_hold)
 {
 	tinfo_key = &add::tinfo_static;
 	overall_coeff = _ex0;
-	construct_from_exvector(v, hold);
+	construct_from_exvector(v, do_hold);
 	GINAC_ASSERT(is_canonical());
 }
 
@@ -95,14 +95,14 @@ add::add(const epvector & v, const ex & oc)
 	GINAC_ASSERT(is_canonical());
 }
 
-add::add(std::auto_ptr<epvector> vp, const ex & oc)
-{
-	tinfo_key = &add::tinfo_static;
-	GINAC_ASSERT(vp.get()!=0);
-	overall_coeff = oc;
-	construct_from_epvector(*vp);
-	GINAC_ASSERT(is_canonical());
-}
+//add::add(std::unique_ptr<epvector> vp, const ex & oc)
+//{
+//	tinfo_key = &add::tinfo_static;
+//	GINAC_ASSERT(vp.get()!=0);
+//	overall_coeff = oc;
+//	construct_from_epvector(*vp);
+//	GINAC_ASSERT(is_canonical());
+//}
 
 //////////
 // archiving
@@ -125,21 +125,19 @@ void add::print_add(const print_context & c, unsigned level, bool latex) const
 			c.s << '(';
 	}
 
-	numeric coeff;
 	bool first = true;
 
 	const epvector & sorted_seq = get_sorted_seq();
 	// Then proceed with the remaining factors
-	epvector::const_iterator it = sorted_seq.begin(), itend = sorted_seq.end();
-	while (it != itend) {
+	for (const auto & elem : sorted_seq) {
 		std::stringstream tstream;
-		print_context *tcontext_p;
+		std::unique_ptr<print_context> tcontext_p;
 		if (latex) {
-			tcontext_p = new print_latex(tstream, c.options);
+			tcontext_p.reset(new print_latex(tstream, c.options));
 		} else {
-			tcontext_p = new print_dflt(tstream, c.options);
+			tcontext_p.reset(new print_dflt(tstream, c.options));
 		}
-		mul(it->rest,it->coeff).print(*tcontext_p, precedence());
+		mul(elem.rest, elem.coeff).print(*tcontext_p, precedence());
 
 		if (!first) {
 			if (tstream.peek() == '-') {
@@ -151,19 +149,17 @@ void add::print_add(const print_context & c, unsigned level, bool latex) const
 			first = false;
 		}
 		tstream.get(*(c.s.rdbuf()));
-		delete tcontext_p;
-		++it;
 	}
 
 	// Finally print the "overall" numeric coefficient, if present.
 	// This is just the constant coefficient. 
 	if (!(ex_to<numeric>(overall_coeff)).is_zero()) {
 		std::stringstream tstream;
-		print_context *tcontext_p;
+		std::unique_ptr<print_context> tcontext_p;
 		if (latex) {
-			tcontext_p = new print_latex(tstream, c.options);
+			tcontext_p.reset(new print_latex(tstream, c.options));
 		} else {
-			tcontext_p = new print_dflt(tstream, c.options);
+			tcontext_p.reset(new print_dflt(tstream, c.options));
 		}
 		overall_coeff.print(*tcontext_p, 0);
 		if (!first) {
@@ -175,7 +171,6 @@ void add::print_add(const print_context & c, unsigned level, bool latex) const
 		}
 
 		tstream.get(*(c.s.rdbuf()));
-		delete tcontext_p;
 	}
 
 	if (precedence() <= level) {
@@ -202,30 +197,28 @@ void add::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << "(";
 	
 	// Print arguments, separated by "+" or "-"
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
 	char separator = ' ';
-	while (it != itend) {
+        for (const auto & elem : seq) {
 		
 		// If the coefficient is negative, separator is "-"
-		if (it->coeff.is_equal(_ex_1) || 
-			ex_to<numeric>(it->coeff).numer().is_equal(*_num_1_p))
+		if (elem.coeff.is_equal(_ex_1) || 
+			ex_to<numeric>(elem.coeff).numer().is_equal(*_num_1_p))
 			separator = '-';
 		c.s << separator;
-		if (it->coeff.is_equal(_ex1) || it->coeff.is_equal(_ex_1)) {
-			it->rest.print(c, precedence());
-		} else if (ex_to<numeric>(it->coeff).numer().is_equal(*_num1_p) ||
-				 ex_to<numeric>(it->coeff).numer().is_equal(*_num_1_p))
+		if (elem.coeff.is_equal(_ex1) || elem.coeff.is_equal(_ex_1)) {
+			elem.rest.print(c, precedence());
+		} else if (ex_to<numeric>(elem.coeff).numer().is_equal(*_num1_p) ||
+				 ex_to<numeric>(elem.coeff).numer().is_equal(*_num_1_p))
 		{
-			it->rest.print(c, precedence());
+			elem.rest.print(c, precedence());
 			c.s << '/';
-			ex_to<numeric>(it->coeff).denom().print(c, precedence());
+			ex_to<numeric>(elem.coeff).denom().print(c, precedence());
 		} else {
-			it->coeff.print(c, precedence());
+			elem.coeff.print(c, precedence());
 			c.s << '*';
-			it->rest.print(c, precedence());
+			elem.rest.print(c, precedence());
 		}
 		
-		++it;
 		separator = '+';
 	}
 	
@@ -240,7 +233,7 @@ void add::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << ")";
 }
 
-void add::do_print_python_repr(const print_python_repr & c, unsigned level) const
+void add::do_print_python_repr(const print_python_repr & c, unsigned) const
 {
 	c.s << class_name() << '(';
 	op(0).print(c);
@@ -270,22 +263,21 @@ bool add::info(unsigned inf) const
 		case info_flags::even:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function: {
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				if (!(recombine_pair_to_ex(*i).info(inf)))
+			for (const auto & elem : seq) {
+				if (!(recombine_pair_to_ex(elem).info(inf)))
 					return false;
-				++i;
 			}
 			if (overall_coeff.is_zero() && (inf == info_flags::positive || inf == info_flags::posint))
-				return true;
-			return overall_coeff.info(inf);
+                                return true;
+                        return overall_coeff.info(inf);
 		}
+		case info_flags::inexact:
 		case info_flags::algebraic: {
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				if ((recombine_pair_to_ex(*i).info(inf)))
+                        if (overall_coeff.info(inf))
+                                return true;
+                        for (const auto & elem : seq) {
+				if ((recombine_pair_to_ex(elem).info(inf)))
 					return true;
-				++i;
 			}
 			return false;
 		}
@@ -295,8 +287,8 @@ bool add::info(unsigned inf) const
 
 bool add::is_polynomial(const ex & var) const
 {
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
-		if (!(i->rest).is_polynomial(var)) {
+	for (const auto & elem : seq) {
+		if (!(elem.rest).is_polynomial(var)) {
 			return false;
 		}
 	}
@@ -310,12 +302,10 @@ int add::degree(const ex & s) const
 		deg = 0;
 	
 	// Find maximum of degrees of individual terms
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		int cur_deg = i->rest.degree(s);
+        for (const auto & elem : seq) {
+		int cur_deg = elem.rest.degree(s);
 		if (cur_deg > deg)
 			deg = cur_deg;
-		++i;
 	}
 	return deg;
 }
@@ -327,40 +317,36 @@ int add::ldegree(const ex & s) const
 		deg = 0;
 	
 	// Find minimum of degrees of individual terms
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		int cur_deg = i->rest.ldegree(s);
+        for (const auto & elem : seq) {
+		int cur_deg = elem.rest.ldegree(s);
 		if (cur_deg < deg)
 			deg = cur_deg;
-		++i;
 	}
 	return deg;
 }
 
 ex add::coeff(const ex & s, int n) const
 {
-	std::auto_ptr<epvector> coeffseq(new epvector);
-	std::auto_ptr<epvector> coeffseq_cliff(new epvector);
+	epvector coeffseq;
+	epvector coeffseq_cliff;
 	int rl = clifford_max_label(s);
 	bool do_clifford = (rl != -1);
 	bool nonscalar = false;
 
 	// Calculate sum of coefficients in each term
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		ex restcoeff = i->rest.coeff(s, n);
+        for (const auto & elem : seq) {
+		ex restcoeff = elem.rest.coeff(s, n);
  		if (!restcoeff.is_zero()) {
  			if (do_clifford) {
  				if (clifford_max_label(restcoeff) == -1) {
- 					coeffseq_cliff->push_back(combine_ex_with_coeff_to_pair(ncmul(restcoeff, dirac_ONE(rl)), i->coeff));
+ 					coeffseq_cliff.push_back(combine_ex_with_coeff_to_pair(ncmul(restcoeff, dirac_ONE(rl)), elem.coeff));
 				} else {
- 					coeffseq_cliff->push_back(combine_ex_with_coeff_to_pair(restcoeff, i->coeff));
+ 					coeffseq_cliff.push_back(combine_ex_with_coeff_to_pair(restcoeff, elem.coeff));
 					nonscalar = true;
  				}
 			}
-			coeffseq->push_back(combine_ex_with_coeff_to_pair(restcoeff, i->coeff));
+			coeffseq.push_back(combine_ex_with_coeff_to_pair(restcoeff, elem.coeff));
 		}
-		++i;
 	}
 
 	return (new add(nonscalar ? coeffseq_cliff : coeffseq,
@@ -376,21 +362,19 @@ ex add::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex add::eval(int level) const
 {
-	std::auto_ptr<epvector> evaled_seqp = evalchildren(level);
-	if (evaled_seqp.get()) {
+	std::unique_ptr<epvector> evaled_seqp = evalchildren(level);
+	if (unlikely(evaled_seqp.get() != nullptr)) {
 		// do more evaluation later
-		return (new add(evaled_seqp, overall_coeff))->
+		return (new add(*evaled_seqp, overall_coeff))->
 		       setflag(status_flags::dynallocated);
 	}
 	
 #ifdef DO_GINAC_ASSERT
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		GINAC_ASSERT(!is_exactly_a<add>(i->rest));
-		if (is_exactly_a<numeric>(i->rest))
+        for (const auto & elem : seq) {
+		GINAC_ASSERT(!is_exactly_a<add>(elem.rest));
+		if (is_exactly_a<numeric>(elem.rest))
 			dbgprint();
-		GINAC_ASSERT(!is_exactly_a<numeric>(i->rest));
-		++i;
+		GINAC_ASSERT(!is_exactly_a<numeric>(elem.rest));
 	}
 #endif // def DO_GINAC_ASSERT
 	
@@ -401,9 +385,9 @@ ex add::eval(int level) const
 	}
 		
 	// handle infinity
-	for (epvector::const_iterator i = seq.begin(); i != seq.end(); i++)
-		if (unlikely(is_exactly_a<infinity>(i->rest)))
-			return eval_infinity(i);
+        for (auto i = seq.begin(); i != seq.end(); i++)
+                if (unlikely(is_exactly_a<infinity>(i->rest)))
+                        return eval_infinity(i);
 
 	/** Perform automatic term rewriting rules */
 	int seq_size = seq.size();
@@ -420,18 +404,18 @@ ex add::eval(int level) const
 	// if any terms in the sum still are purely numeric, then they are more
 	// appropriately collected into the overall coefficient
 	int terms_to_collect = 0;
-	for (epvector::const_iterator j = seq.begin(); j != seq.end(); j++)
-		if (unlikely(is_exactly_a<numeric>(j->rest)))
+	for (const auto & elem : seq)
+		if (unlikely(is_exactly_a<numeric>(elem.rest)))
 			++terms_to_collect;
 	if (terms_to_collect) {
-		std::auto_ptr<epvector> s(new epvector);
-		s->reserve(seq_size - terms_to_collect);
+                epvector s;
+		s.reserve(seq_size - terms_to_collect);
 		numeric oc = *_num0_p;
-		for (epvector::const_iterator j = seq.begin(); j != seq.end(); j++)
-			if (unlikely(is_exactly_a<numeric>(j->rest)))
-				oc = oc.add((ex_to<numeric>(j->rest)).mul(ex_to<numeric>(j->coeff)));
+                for (const auto & elem : seq)
+			if (unlikely(is_exactly_a<numeric>(elem.rest)))
+				oc = oc.add((ex_to<numeric>(elem.rest)).mul(ex_to<numeric>(elem.coeff)));
 			else
-				s->push_back(*j);
+				s.push_back(elem);
 		return (new add(s, ex_to<numeric>(overall_coeff).add_dyn(oc)))
 		        ->setflag(status_flags::dynallocated);
 	}
@@ -458,7 +442,7 @@ ex add::eval_infinity(epvector::const_iterator infinity_iter) const
 	GINAC_ASSERT(is_exactly_a<infinity>(infinity_iter->rest));
 	infinity result = infinity_from_iter(infinity_iter);
 
-        for (epvector::const_iterator i = seq.begin(); i != seq.end(); i++) {
+        for (auto i = seq.begin(); i != seq.end(); i++) {
                 if (not is_exactly_a<infinity>(i->rest)) continue;
                 if (i == infinity_iter) continue;
 		infinity i_infty = infinity_from_iter(i);
@@ -472,17 +456,16 @@ ex add::evalm() const
 {
 	// Evaluate children first and add up all matrices. Stop if there's one
 	// term that is not a matrix.
-	std::auto_ptr<epvector> s(new epvector);
-	s->reserve(seq.size());
+	epvector s;
+	s.reserve(seq.size());
 
 	bool all_matrices = true;
 	bool first_term = true;
 	matrix sum;
 
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		const ex &m = recombine_pair_to_ex(*it).evalm();
-		s->push_back(split_ex_to_pair(m));
+	for (const auto & elem : seq) {
+		const ex &m = recombine_pair_to_ex(elem).evalm();
+		s.push_back(split_ex_to_pair(m));
 		if (is_exactly_a<matrix>(m)) {
 			if (first_term) {
 				sum = ex_to<matrix>(m);
@@ -491,7 +474,6 @@ ex add::evalm() const
 				sum = sum.add(ex_to<matrix>(m));
 		} else
 			all_matrices = false;
-		++it;
 	}
 
 	if (all_matrices)
@@ -502,7 +484,7 @@ ex add::evalm() const
 
 ex add::conjugate() const
 {
-	exvector *v = 0;
+	std::unique_ptr<exvector> v(nullptr);
 	for (size_t i=0; i<nops(); ++i) {
 		if (v) {
 			v->push_back(op(i).conjugate());
@@ -512,7 +494,7 @@ ex add::conjugate() const
 		ex ccterm = term.conjugate();
 		if (are_ex_trivially_equal(term, ccterm))
 			continue;
-		v = new exvector;
+		v.reset(new exvector);
 		v->reserve(nops());
 		for (size_t j=0; j<i; ++j)
 			v->push_back(op(j));
@@ -520,7 +502,6 @@ ex add::conjugate() const
 	}
 	if (v) {
 		ex result = add(*v);
-		delete v;
 		return result;
 	}
 	return *this;
@@ -530,13 +511,13 @@ ex add::real_part() const
 {
 	epvector v;
 	v.reserve(seq.size());
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
-		if ((i->coeff).info(info_flags::real)) {
-			ex rp = (i->rest).real_part();
+	for (const auto & elem : seq)
+		if ((elem.coeff).info(info_flags::real)) {
+			ex rp = (elem.rest).real_part();
 			if (!rp.is_zero())
-				v.push_back(expair(rp, i->coeff));
+				v.push_back(expair(rp, elem.coeff));
 		} else {
-			ex rp=recombine_pair_to_ex(*i).real_part();
+			ex rp=recombine_pair_to_ex(elem).real_part();
 			if (!rp.is_zero())
 				v.push_back(split_ex_to_pair(rp));
 		}
@@ -548,13 +529,13 @@ ex add::imag_part() const
 {
 	epvector v;
 	v.reserve(seq.size());
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
-		if ((i->coeff).info(info_flags::real)) {
-			ex ip = (i->rest).imag_part();
+	for (const auto & elem : seq)
+		if ((elem.coeff).info(info_flags::real)) {
+			ex ip = (elem.rest).imag_part();
 			if (!ip.is_zero())
-				v.push_back(expair(ip, i->coeff));
+				v.push_back(expair(ip, elem.coeff));
 		} else {
-			ex ip=recombine_pair_to_ex(*i).imag_part();
+			ex ip=recombine_pair_to_ex(elem).imag_part();
 			if (!ip.is_zero())
 				v.push_back(split_ex_to_pair(ip));
 		}
@@ -576,17 +557,14 @@ ex add::eval_ncmul(const exvector & v) const
  *  @see ex::diff */
 ex add::derivative(const symbol & y) const
 {
-	std::auto_ptr<epvector> s(new epvector);
-	s->reserve(seq.size());
+	epvector s;
+	s.reserve(seq.size());
 	
 	// Only differentiate the "rest" parts of the expairs. This is faster
 	// than the default implementation in basic::derivative() although
 	// if performs the same function (differentiate each term).
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		s->push_back(combine_ex_with_coeff_to_pair(i->rest.diff(y), i->coeff));
-		++i;
-	}
+	for (const auto & elem : seq)
+		s.push_back(combine_ex_with_coeff_to_pair(elem.rest.diff(y), elem.coeff));
 	return (new add(s, _ex0))->setflag(status_flags::dynallocated);
 }
 
@@ -612,15 +590,15 @@ tinfo_t add::return_type_tinfo() const
 }
 
 // Note: do_index_renaming is ignored because it makes no sense for an add.
-ex add::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
+ex add::thisexpairseq(const epvector & v, const ex & oc, bool) const
 {
 	return (new add(v,oc))->setflag(status_flags::dynallocated);
 }
 
-// Note: do_index_renaming is ignored because it makes no sense for an add.
-ex add::thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming) const
+//// Note: do_index_renaming is ignored because it makes no sense for an add.
+ex add::thisexpairseq(std::unique_ptr<epvector> vp, const ex & oc, bool) const
 {
-	return (new add(vp,oc))->setflag(status_flags::dynallocated);
+	return (new add(*vp,oc))->setflag(status_flags::dynallocated);
 }
 
 expair add::split_ex_to_pair(const ex & e) const
@@ -628,7 +606,9 @@ expair add::split_ex_to_pair(const ex & e) const
 	if (is_exactly_a<mul>(e)) {
 		const mul &mulref(ex_to<mul>(e));
 		const ex &numfactor = mulref.overall_coeff;
-		mul *mulcopyp = new mul(mulref);
+		if (numfactor.is_integer_one())
+                        return expair(e, _ex1);
+                auto mulcopyp = new mul(mulref);
 		mulcopyp->overall_coeff = _ex1;
 		mulcopyp->clearflag(status_flags::evaluated);
 		mulcopyp->clearflag(status_flags::hash_calculated);
@@ -645,20 +625,22 @@ expair add::combine_ex_with_coeff_to_pair(const ex & e,
 	if (is_exactly_a<mul>(e)) {
 		const mul &mulref(ex_to<mul>(e));
 		const ex &numfactor = mulref.overall_coeff;
-		mul *mulcopyp = new mul(mulref);
+		if (numfactor.is_integer_one())
+                        return expair(e, c);
+		auto mulcopyp = new mul(mulref);
 		mulcopyp->overall_coeff = _ex1;
 		mulcopyp->clearflag(status_flags::evaluated);
 		mulcopyp->clearflag(status_flags::hash_calculated);
 		mulcopyp->setflag(status_flags::dynallocated);
-		if (c.is_equal(_ex1))
+		if (c.is_integer_one())
 			return expair(*mulcopyp, numfactor);
-		else if (numfactor.is_equal(_ex1))
-			return expair(*mulcopyp, c);
 		else
 			return expair(*mulcopyp, ex_to<numeric>(numfactor).mul_dyn(ex_to<numeric>(c)));
 	} else if (is_exactly_a<numeric>(e)) {
-		if (c.is_equal(_ex1))
+		if (c.is_integer_one())
 			return expair(e, _ex1);
+		if (e.is_integer_one())
+			return expair(c, _ex1);
 		return expair(ex_to<numeric>(e).mul_dyn(ex_to<numeric>(c)), _ex1);
 	}
 	return expair(e, c);
@@ -688,13 +670,13 @@ ex add::recombine_pair_to_ex(const expair & p) const
 
 ex add::expand(unsigned options) const
 {
-	std::auto_ptr<epvector> vp = expandchildren(options);
-	if (vp.get() == 0) {
+	std::unique_ptr<epvector> vp = expandchildren(options);
+	if (vp.get() == nullptr) {
 		// the terms have not changed, so it is safe to declare this expanded
 		return (options == 0) ? setflag(status_flags::expanded) : *this;
 	}
 
-	return (new add(vp, overall_coeff))->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
+	return (new add(*vp, overall_coeff))->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
 }
 
 const epvector & add::get_sorted_seq() const
