@@ -367,6 +367,8 @@ epvector* conjugateepvector(const epvector& epv)
                 for (const auto & elem2 : epv)
                         if (&elem2 != &elem)
                                 newepv->push_back(elem2);
+                        else
+                                break;
 		newepv->push_back(x);
 	}
 	return newepv;
@@ -640,9 +642,9 @@ unsigned expairseq::return_type() const
 	return return_types::noncommutative_composite;
 }
 
-unsigned expairseq::calchash() const
+long expairseq::calchash() const
 {
-	unsigned v = golden_ratio_hash((p_int)this->tinfo());
+	long v = golden_ratio_hash((p_int)this->tinfo());
         for (const auto & elem : seq) {
 		v ^= elem.rest.gethash();
 #if !EXPAIRSEQ_USE_HASHTAB
@@ -1644,7 +1646,7 @@ std::unique_ptr<epvector> expairseq::evalchildren(int level) const
 	// returns a pointer to a newly created epvector otherwise
 	// (which has to be deleted somewhere else)
 
-	if (level==1)
+	if (likely(level==1))
 		return std::unique_ptr<epvector>(nullptr);
 	
 	if (level == -max_recursion_level)
